# Generated by Django 2.2.15 on 2020-09-04 03:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workflow', '0003_TeamWorkflows'),
    ]

    operations = [
        migrations.AddField(
            model_name='assessmentworkflowstep',
            name='skipped',
            field=models.BooleanField(default=False, null=True),
        ),
    ]
