{% load i18n %}
{% spaceless %}
<li class="field comp-setting-entry openassessment_training_example_criterion" data-criterion="{{ criterion.name }}">
    <div class="wrapper-comp-setting">
        <label class="openassessment_training_example_criterion_name setting-label">
            <div class="openassessment_training_example_criterion_name_wrapper">
                {{ criterion.label }}
            </div>
            <select class="openassessment_training_example_criterion_option setting-input" data-criterion="{{ criterion.name }}" data-option="{{ option.name }}">
                <option value="">{% trans "Not Selected" %}</option>
                {% for option in criterion.options %}
                    <option value="{{ option.name }}" data-points="{{ option.points }}" data-label="{{ option.label }}"
                                    {% if criterion.option_selected == option.name %} selected {% endif %}
                            >
                        {{ option.label }} - {{ option.points }} {% trans "points" %}
                    </option>
                {% endfor %}
            </select>
        </label>
    </div>
</li>
{% endspaceless %}