# Generated by Django 1.11.26 on 2020-01-03 20:50


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('assessment', '0004_historicalsharedfileupload_sharedfileupload'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalsharedfileupload',
            name='name',
            field=models.CharField(default='', max_length=255),
        ),
        migrations.AddField(
            model_name='sharedfileupload',
            name='name',
            field=models.CharField(default='', max_length=255),
        ),
    ]
