{% extends "openassessmentblock/peer/oa_peer_assessment.html" %}
{% load i18n %}

{% block list_item %}
<li id="openassessment__peer-assessment__{{ xblock_id }}" class="openassessment__steps__step step--peer-assessment is--incomplete is--showing ui-slidable__container" tabindex="-1">
{% endblock %}


{% block button %}
    <button class="ui-slidable" aria-expanded="true" id="oa_peer_{{ xblock_id }}" aria-controls="oa_peer_{{ xblock_id }}_content" aria-labeledby="oa_step_title_peer">
        <span class="icon fa fa-caret-right" aria-hidden="false"/>
    </button>
{% endblock %}

{% block title %}
<span class="step__status">
  <span id="oa_step_status_peer" class="step__status__value">
    <span class="copy">
      {% with review_num_string=review_num|stringformat:"s" must_grade_string=must_grade|stringformat:"s" %}
        {% blocktrans with review_number='<span class="step__status__number--current">'|safe|add:review_num_string|add:"</span>"|safe num_must_grade='<span class="step__status__value--required">'|safe|add:must_grade_string|add:"</span>"|safe %}
          In Progress ({{ review_number }} of {{ num_must_grade }})
        {% endblocktrans %}
      {% endwith %}
    </span>
  </span>
</span>
{% endblock %}

{% block body %}
<div role="region" class="ui-slidable__content" aria-labelledby="oa_peer_{{ xblock_id }}" id="oa_peer_{{ xblock_id }}_content">
    <div class="wrapper--step__content">

        <div class="step__message message message--incomplete">
            <h5 class="message__title">{% trans "Status" %}</h5>

            <div class="message__content">
              <p>{% trans "All available peer responses have been assessed. Check back later to see if more learners have submitted responses. You will receive your grade after you've completed all the steps for this problem and your peers have assessed your response." %}</p>
            </div>
        </div>
    </div>
</div>
{% endblock %}
